// Core - Bridging

const _nb_instance_pool = {};

const _nb_callback = {
  callbackID: 1,
  callbacks: {},
};

function callNative(module, method, params, callback) {
  let callbackId = _nb_callback.callbackID + '';
  let call = { module, method, callbackId, params };
  if (callback) {
    _nb_callback.callbacks[callbackId] = callback;
    _nb_callback.callbackID = _nb_callback.callbackID + 1;
  }
  nb_js_bridge_callNative(JSON.stringify(call));
}

/**
 *  JS Bridge callback by nb_js_bridge_callback
 *
 *  @callbackId passby
 *  @success  jsapi run result
 *  @params jsapi params
 */
function nb_js_bridge_callback(callbackId, success, params) {
  if (!callbackId) {
    return;
  }
  let callback = _nb_callback.callbacks[callbackId];
  if (callback) {
    // log('nb_js_bridge_callback find callback:' + callbackId);
    let jsonParams = {};
    if (params) {
      // need to decode url params to json string
      let decoded = decodeURIComponent(params);
      // parse json string to object
      let jsonData = null;
      try {
        jsonParams = JSON.parse(decoded);
      } catch (e) {
        log('nb_js_bridge_callback parse json exception: ' + result);
      }
    }
    callback(jsonParams);
  } else {
    log('nb_js_bridge_callback not find callback: ' + callbackId);
  }
  // delete callback
  delete _nb_callback.callbacks[callbackId];
}

/**
 *    调用jsapi格式: nb_js_bridge_callNative('json字符串')， json包括
 *    @module： 模块名
 *    @method： 方法名
 *    @callbackId： 回调ID，执行成功后，会调用nb_js_bridge_callback方法，并透传回来callbackId
 *    @params 方法参数
 *    jsapi参见： https://iwiki.woa.com/pages/viewpage.action?pageId=302119223
 *
 *    示例：nb_js_bridge_callNative('{"module": "interaction", "method":"showToast", "callbackId":"12345","params": {"":1, "b":2}}');
 */

// Support console.log

const LOG_LEVEL = {
  Info: 0,
  Debug: 1,
  Warning: 2,
  Error: 3,
};

const console = {
  log: function (...params) {
    log(params.map((param) => JSON.stringify(param))
      .join(' '));
  },
  error: function (...params) {
    log(params.map((param) => JSON.stringify(param))
      .join(' '), LOG_LEVEL.Error);
  },
};

function log(log, level) {
  if (!log) {
    return;
  }
  let params = {
    level: level || LOG_LEVEL.Debug,
    tag: 'JSAPI',
    log: log,
  };
  callNative('log', 'log', params);
}

// window.setTimeout
function setTimeout(fn, delay) {
  fn();
}

function showToast(msg) {
  let callback = (params) => {
    log('showToast callback code:' + params.code + ' msg: ' + params.msg);
  };
  callNative('interaction', 'showToast', `{"title":"${msg}"}`, callback);
}

// Global scroll method for an AresView
function scrollBy(x, y) {
  callNative('ares', 'scrollBy', { 'x': x, 'y': y }, null);
}

function getAppVersion(callback) {
  callNative('ares', 'getAppVersion', {}, callback);
}

function openRouterUrl(url) {
  if (!url) {
    return;
  }

  let json = '{"url":"' + url + '"}';
  callNative('router', 'navigateTo', json, (jsonData) => {
    log('openRouterUrl code: ' + jsonData.code + ' msg: ' + jsonData.msg);
  });
}

class CommonBusinessScripts {
  static onOpenProductItem(data) {
    let params = {
      url: 'yz://product-detail',
      data: data,
    };
    callNative('router', 'navigateTo', JSON.stringify(params), null);
  }

  static formatPrice(priceCent) {
    if (priceCent % 100 == 0) {
      return (priceCent / 100).toFixed(0);
    } else if (priceCent % 10 == 0) {
      return (priceCent / 100).toFixed(1);
    }
    return (priceCent / 100).toFixed(2);
  }

  static spuImage(spu) {
    if (spu) {
      let image = spu.imageUrl || spu.ImgUrl;
      if (typeof image === 'object' && image.length && image.length > 0) {
        return image[0];
      }
    }
    return image || '';
  }

  static getPriceDecimal(price) {
    if (price % 100 === 0) {
      return '';
    }
    return ((price % 100) / 100).toFixed(2).toString().substring(1);
  }
}
